@@ -31,6 +31,7 @@ DIDparams <- function(yname,
                    clustervars=NULL,
                    cband=TRUE,
                    print_details=TRUE,
+                   faster_mode=FALSE,
                    pl=FALSE,
                    cores=1,
                    est_method="dr",
@@ -59,6 +60,7 @@ DIDparams <- function(yname,
               clustervars=clustervars,
               cband=cband,
               print_details=print_details,
+              faster_mode=faster_mode,
               pl=pl,
               cores=cores,
               est_method=est_method,
