@@ -24,7 +24,9 @@
 #' @return MP object
 #' @export
 MP <- function(group, t, att, V_analytical, se, c, inffunc, n=NULL, W=NULL, Wpval=NULL, aggte=NULL, alp = 0.05, DIDparams=NULL) {
-  out <- list(group=group, t=t, att=att, V_analytical=V_analytical, se=se, c=c, inffunc=inffunc, n=n, W=W, Wpval=Wpval, aggte=aggte, alp = alp, DIDparams=DIDparams)
+  out <- list(group=group, t=t, att=att, V_analytical=V_analytical, se=se, c=c,
+  inffunc=inffunc, n=n, W=W, Wpval=Wpval, aggte=aggte, alp = alp,
+  DIDparams=DIDparams, call=DIDparams$call)
   class(out) <- "MP"
   out
 }
@@ -108,7 +110,7 @@ summary.MP <- function(object, ...) {
 
   # estimation method text
   est_method <- mpobj$DIDparams$est_method
-  if (class(est_method)=="character") {
+  if ( is(est_method,"character") ) {
     est_method_text <- est_method
     if (est_method == "dr") {
       est_method_text <- "Doubly Robust"
